#!/usr/bin/python
# -*- coding: utf-8 -*-
"""
 dnscheck.py : Program parses command line arguments and calls
 modules according to their dependencies

 Copyright (c) 2009, Karel Slany    (slany AT fit.vutbr.cz)
                     Zdenek Vasicek (vasicek AT fit.vutbr.cz)
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of the organization nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
"""

import sys;sys.path.append("../ldns-src/contrib/python/examples")
import modules

import optparse
import locale
import os
import encodings.idna

#parse arguments
oparser = optparse.OptionParser()
oparser.add_option("-d", "--dname", action="store",
                   dest="dname", metavar="DOMAIN",
                   help="set domain name")
oparser.add_option("-n", "--noipv4", action="store_true",
                   dest="noipv4",
                   help="disable IPv4 checks")
oparser.add_option("-6", "--ipv6", action="store_true",
                   dest="ipv6",
                   help="enable IPv6 checks - does not create IPv6 resolvers")
oparser.add_option("-r", "--resolver6", action="store_true",
                   dest="resolver6",
                   help="enable IPv6 checks - creates IPv6 based resolvers")
oparser.add_option("-R", "--vresolver", action="append",
                   dest="vresolver", metavar="VALIDATING-RESOLVER-IP",
                   help="use a validating resolver to resolve IPs")
oparser.add_option('-m', '--mxrecs', action="store_true",
                   dest="mxrecs",
                   help="show MX records")
oparser.add_option('-M', "--mailcheck", action="store_true",
                   dest="mailcheck",
                   help="checks mailservers")
oparser.add_option('-a', '--arecs', action="append",
                   dest="arecs", metavar="NAME",
                   help="show A records for names in domain, this option may be used multiple times")
oparser.add_option('-w', "--web", action="store_true",
                   dest="web",
                   help="check machines listed by A record parameters for HTTP server presence")
oparser.add_option('-z', '--zone', action="store_true",
                   dest="zone",
                   help="check zone download")
oparser.add_option("-v", "--verbose", action="count",
                   dest="verbosity",
                   help="increase verbosity, may be used multiple times")
oparser.set_defaults(noipv4=False,
                     ipv6=False,
                     resolver6=False,
                     mxrecs=False,
                     mailcheck = False,
                     http=False,
                     verbosity=0)
(options, args) = oparser.parse_args()

lang, enc = os.environ.get("LANG", '').split(".")
loc_lang, loc_enc = locale.getdefaultlocale()
if (enc != loc_enc):
   print "Warning: environment variable LANG="+os.environ.get("LANG", '')+" differs from the value obtained from default locale "+".".join(locale.getdefaultlocale())
   print "         using "+loc_enc
   print ""

def argument2utf(str):
    return unicode(str, loc_enc)

def utf2name(name):
    splited = name.split('.')
    while (u"" in splited):
        splited.remove(u"")
    restored = '.'.join([encodings.idna.ToASCII(a) for a in splited])
    return restored + "."

def name6utf(name):
    splited = name.split('.')
    while (u"" in splited):
        splited.remove(u"")
    restored = '.'.join([encodings.idna.ToUnicode(a) for a in splited])
    return restored + "."

# test whether IPv6 functionality is set correct
if (options.resolver6):
    options.ipv6 = True

# test for mailservers implies
if (options.mailcheck):
    options.mxrecs = True

# http option has to have also arecs option set
if (options.web and (not options.arecs)):
    oparser.print_help()
    print "\nIf you want to test for HTTP presence, do not forget to give names via the A records option."
    sys.exit(1)

# validating resolver works in the A module
if (options.vresolver and (not options.arecs)):
    oparser.print_help()
    print "\nIf you want to use a validating resolver, then please use it in the combination with -a option."
    sys.exit(1)

# create domain names for A requests
if (options.arecs):
    a_list = map(lambda x : utf2name(argument2utf(x+"."+options.dname)), options.arecs)
    options.arecs = a_list

# convert domain name to IDNA
if (options.dname):
   options.dname = utf2name(argument2utf(options.dname))

class CmdArg:
    def __init__(self, options, args):
        self.options = options
        self.args = args

# arguments
args = CmdArg(options, args)

if (args.options.dname == None):
    oparser.print_help()
    print "\nPlease specify domain to be checked."
    sys.exit(1)

# environment
modenv = {}

# call modules
for m in modules.get_module_list(options.verbosity > 1):
    mm = m(args, modenv)

    if (options.verbosity):
       print "Module:",mm.module_name()
       print "========================================================"

    if not mm.execute():
        raise Exception("Module "+ mm.module_name()+": error")

    if (options.verbosity):
       print
