"""
 interface.py : Base module class definition

 Copyright (c) 2009, Karel Slany    (slany AT fit.vutbr.cz)
                     Zdenek Vasicek (vasicek AT fit.vutbr.cz)
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met:
 
     * Redistributions of source code must retain the above copyright notice,
       this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of the organization nor the names of its
       contributors may be used to endorse or promote products derived from this
       software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
"""

import sys
import types
import copy
import ldns

class TestModule:
    def __init__(self, args, modenv, modname):
        """Init module"""
        self.modname = modname
        self.modenv = modenv
        self.args = args
        self.ldns = ldns

    def execute(self):
        """Execute module
    
           The result of module should be stored in self.modenv dictionary

           :returns: False if operation failed
        """
        raise Exception("module "+self.modname+": Execute method not implemented yet")

    def log(self, verbosity, *args):
        """Print log message"""
        if (verbosity <= self.args.options.verbosity):
            print "   "+self.modname+": ",
            for a in args:
                print a,
            print

    def verb(self, verbosity, *args):
        """Print verbose message"""
        if (verbosity <= self.args.options.verbosity):
            for a in args:
                print a,
            print

    def err(self, verbosity, *args):
        """Print error message"""
        if (verbosity <= self.args.options.verbosity):
            sys.stderr.write("   "+self.modname+" error: ")
            for a in args:
                sys.stderr.write(str(a))
            sys.stderr.write("\n")

    def module_name(self):
        """Get a name of the module"""
        return self.modname

    def get_module_modenv(self, moduleobj):
        """Get the content of sandbox for a given module"""
        assert isinstance(moduleobj,types.ModuleType)
        a = copy.copy(self.args)
        e = copy.copy(self.modenv)
        modname = moduleobj.Module(a,e).module_name()
        if self.modenv and (modname in self.modenv):
            return self.modenv[modname]
        else:
            self.log("Warning: module",modname,"doesn't produces any result")
            return []

    def get_my_modenv(self):
        """Get the content of my sandbox"""
        if not self.modname in self.modenv:
            return None
        return self.modenv[self.modname]

    def set_my_modenv(self, value):
        """Set the content of my sandbox"""
        #assert isinstance(value,types.DictType)
        self.modenv[self.modname] = value

    my_modenv = property(get_my_modenv, set_my_modenv, None, "The module sandbox");

